#ifndef _LIBXMLPP_CONFIG_H
#define _LIBXMLPP_CONFIG_H

#include <glibmmconfig.h>

/* Define to omit deprecated API from the library. */
/* #undef LIBXMLXX_DISABLE_DEPRECATED */

/* Defined if the C++ library supports std::exception_ptr. */
#define LIBXMLXX_HAVE_EXCEPTION_PTR 1

/* Major version number of libxml++. */
#define LIBXMLXX_MAJOR_VERSION 3

/* Minor version number of libxml++. */
#define LIBXMLXX_MINOR_VERSION 2

/* Micro version number of libxml++. */
#define LIBXMLXX_MICRO_VERSION 6

#if defined (GLIBMM_DLL) && !defined (LIBXMLXX_STATIC)
  #ifdef LIBXMLPP_BUILD
    #define LIBXMLPP_API __declspec(dllexport)
  #elif !defined (__GNUC__)
    #define LIBXMLPP_API __declspec(dllimport)
  #else /* don't dllimport classes/methods on GCC/MinGW */
    #define LIBXMLPP_API
  #endif /* LIBXMLPP_BUILD - DLL */
#else
  /* Build a static library or a non-Windows library*/
  #define LIBXMLPP_API
#endif /* GLIBMM_DLL */

#endif /* _LIBXMLPP_CONFIG_H */
